@@ -7,7 +7,7 @@
 // INCLUDES
 
 #ifdef HAVE_CONFIG_H
-#include "dgconfig.h"
+#include "e2config.h"
 #endif
 #include <csignal>
 #include <string.h>
@@ -17,32 +17,16 @@
 #include <cerrno>
 #include <unistd.h>
 #include <stdexcept>
-#include <syslog.h>
 #include <sys/select.h>
 
-#ifdef NETDEBUG
-#include <iostream>
-#endif
-
 #include "BaseSocket.hpp"
+#include "Logger.hpp"
 
 // GLOBALS
 extern bool reloadconfig;
-extern thread_local std::string thread_id;
 
 // DEFINITIONS
 
-#define dgtimercmp(a, b, cmp) \
-    (((a)->tv_sec == (b)->tv_sec) ? ((a)->tv_usec cmp(b)->tv_usec) : ((a)->tv_sec cmp(b)->tv_sec))
-
-#define dgtimersub(a, b, result)                     \
-    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;    \
-    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec; \
-    if ((result)->tv_usec < 0) {                     \
-        (result)->tv_sec--;                          \
-        (result)->tv_usec += 1000000;                \
-    }
-
 // IMPLEMENTATION
 
 // This class contains client and server socket init and handling
@@ -50,7 +34,6 @@ extern thread_local std::string thread_id;
 
 // constructor - override this if desired to create an actual socket at startup
 BaseSocket::BaseSocket()
-    : timeout(5000), sck(-1), buffstart(0), bufflen(0)
 {
     infds[0].fd = -1;
     outfds[0].fd = -1;
@@ -65,7 +48,6 @@ BaseSocket::BaseSocket()
 
 // create socket from FD - must be overridden to clear the relevant address structs
 BaseSocket::BaseSocket(int fd)
-    : timeout(5000), buffstart(0), bufflen(0)
 {
     sck = fd;
     infds[0].fd = fd;
@@ -98,7 +80,7 @@ void BaseSocket::baseReset()
         infds[0].fd = -1;
         outfds[0].fd = -1;
     }
-    timeout = 5000;
+   // timeout = 5000;   // commented out so that timeout can be set before a connect call
     buffstart = 0;
     bufflen = 0;
     isclosing = false;
@@ -128,11 +110,6 @@ inline int local_accept_adaptor(int (*accept_func)(int, struct sockaddr *, T),
 int BaseSocket::baseAccept(struct sockaddr *acc_adr, socklen_t *acc_adr_length)
 {
 
-    // OS X defines accept as:
-    // int accept(int s, struct sockaddr *addr, int *addrlen);
-    // but everyone else as:
-    // int accept(int s, struct sockaddr *addr, socklen_t *addrlen);
-    // NB: except 10.4, which seems to use the more standard definition. grrr.
     return local_accept_adaptor(::accept, sck, acc_adr, acc_adr_length);
 }
 
@@ -216,21 +193,33 @@ bool BaseSocket::isNoWrite()
     return ( sockerr || ishup || (sck < 0));
 }
 
+
 // blocking check to see if there is data waiting on socket
-bool BaseSocket::bcheckSForInput(int timeout)
+bool BaseSocket::checkForInput(int timeout)
 {
+    if ((bufflen - buffstart) > 0)
+        return true; // is data left in buffer
+
     if (isNoRead())
         return false;
+
+    if(timeout == 0) { // no poll wanted as done by calling function
+         return false;
+    }
+
     int rc;
     s_errno = 0;
     errno = 0;
+
     rc = poll(infds, 1, timeout);
+
     if (rc == 0)
     {
         timedout = true;
         return false;   //timeout
     }
     timedout = false;
+
     if (rc < 0)
     {
         s_errno = errno;
@@ -247,101 +236,8 @@ bool BaseSocket::bcheckSForInput(int timeout)
     return false;   // must be POLLERR or POLLNVAL
 }
 
-// blocking check to see if there is data waiting on socket
-bool BaseSocket::bcheckForInput(int timeout)
-{
-    if ((bufflen - buffstart) > 0)
-        return true;
-    if (isNoRead())
-        return false;
-    int rc;
-    s_errno = 0;
-    errno = 0;
-    rc = poll(infds, 1, timeout);
-    if (rc == 0)
-    {
-        timedout = true;
-        return false;   //timeout
-    }
-    timedout = false;
-    if (rc < 0)
-    {
-        s_errno = errno;
-        sockerr = true;
-        return false;
-    }
-    if (infds[0].revents & POLLHUP) {
-        ishup = true;
-    }
-    if ((infds[0].revents & (POLLHUP | POLLIN))) {
-        return true;
-    }
-    sockerr = true;
-    return false;   // must be POLLERR or POLLNVAL
-}
 
-// blocking check for waiting data - blocks for up to given timeout, can be told to break on signal-triggered config reloads
-bool BaseSocket::checkForInput()
-{
-    if ((bufflen - buffstart) > 0)
-        return true;
-    if (isNoRead())
-        return false;
-    int rc;
-    s_errno = 0;
-    errno = 0;
-   rc = poll(infds, 1, 0);
-    if (rc == 0)
-        {
-        return false;   //timeout
-    }
-    timedout = false;
-    if (rc < 0)
-    {
-        s_errno = errno;
-        sockerr = true;
-        return false;
-    }
-    if (infds[0].revents & POLLHUP) {
-        ishup = true;
-    }
-    if (infds[0].revents & (POLLHUP | POLLIN)) {
-        return true;
-    }
-    sockerr = true;
-    return false;   // must be POLLERR or POLLNVAL
-}
-
-
-
-// non-blocking check to see if a socket is ready to be written     //NOT EVER USED   - not it is used in Socket.cpp
-bool BaseSocket::readyForOutput()
-{
-    if (isNoWrite())
-        return false;
-    int rc;
-    s_errno = 0;
-    errno = 0;
-    rc = poll(outfds,1, 0);
-    if (rc == 0)
-    {
-        return false;
-    }
-    timedout = false;
-    if (rc < 0)
-    {
-        s_errno = errno;
-        sockerr = true;
-        return false;
-    }
-    if (outfds[0].revents & POLLOUT)
-         return true;
-    if (outfds[0].revents & POLLHUP)
-        ishup = true;
-    return false;
-}
-
-bool BaseSocket::breadyForOutput(int timeout) {
+bool BaseSocket::readyForOutput(int timeout) {
     if (isNoWrite())
         return false;
     int rc;
@@ -365,23 +261,21 @@ bool BaseSocket::breadyForOutput(int timeout) {
     return false;
 }
 
-// read a line from the socket, can be told to break on config reloads
-int BaseSocket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig, bool *chopped, bool *truncated) //throw(std::exception)
+// read a line from the socket
+int BaseSocket::getLine(char *buff, int size, int timeout, bool *chopped, bool *truncated)
 {
     try {
         // first, return what's left from the previous buffer read, if anything
         int i = 0;
         if ((bufflen - buffstart) > 0) {
-#ifdef NETDEBUG
-            std::cout << thread_id  << "data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
-#endif
+            DEBUG_network("data already in buffer; bufflen: ", bufflen, " buffstart: ", buffstart);
 
             //work out the maximum size we want to read from our internal buffer
             int tocopy = size - 1;
             if ((bufflen - buffstart) < tocopy)
                 tocopy = bufflen - buffstart;
 
-            //copy the data to output buffer (up to 8192 chars in loglines case)
+            //copy the data to output buffer
             char *result = (char *) memccpy(buff, buffer + buffstart, '\n', tocopy);
 
             //if the result was < max size
@@ -403,32 +297,27 @@ int BaseSocket::getLine(char *buff, int size, int timeout, bool honour_reloadcon
         while (i < (size - 1)) {
             buffstart = 0;
             bufflen = 0;
-//        try {
             s_errno = 0;
             errno = 0;
-            if (bcheckForInput(timeout))
-                bufflen = recv(sck, buffer, 1024, 0);
-            //      } catch (std::exception &e) {
-            //          throw std::runtime_error(std::string("Can't read from socket: ") + e.what()); // on error
-            //     }
-#ifdef NETDEBUG
-            std::cout << thread_id  << "getLine !SSL read into buffer; bufflen: " << bufflen << std::endl;
-#endif
-            //if there was a socket error
+            if (!isNoBlock) {
+                if(!checkForInput(timeout))
+                    return -1;
+            }
+            bufflen = recv(sck, buffer, SCK_READ_BUFF_SIZE, 0);
             if (bufflen < 0) {
-#ifdef NETDEBUG
-                std::cout << thread_id  << "getLine Can't read from socket !SSL: " << std::endl;
-#endif
+                if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+                    if(isNoBlock) {
+                        if (checkForInput(timeout)) continue;// now got some input
+                        else  return -1;// timed out
+                    }
+                }
                 s_errno = errno;
                 return -1;
-//            throw std::runtime_error(std::string("Can't read from socket: ") + strerror(errno)); // on error
             }
             //if socket closed...
             if (bufflen == 0) {
                 buff[i] = '\0'; // ...terminate string & return what read
-#ifdef NETDEBUG
-                std::cout << thread_id  << "getLine terminate string !SSL: " << i << std::endl;
-#endif
+                DEBUG_network("getLine terminate string !SSL: ", i );
                 if (truncated)
                     *truncated = true;
                 return i;
@@ -438,17 +327,13 @@ int BaseSocket::getLine(char *buff, int size, int timeout, bool honour_reloadcon
                 tocopy = (size - 1) - i;
             char *result = (char *) memccpy(buff + i, buffer, '\n', tocopy);
             if (result != NULL) {
-#ifdef NETDEBUG
-                std::cout << thread_id  << "getLine result1 !SSL: " << result << i << std::endl;
-#endif
+                DEBUG_network("getLine result1 !SSL: ", result, i );
                 // indicate that a newline was chopped off, if desired
                 if (chopped)
                     *chopped = true;
                 *(--result) = '\0';
                 buffstart += (result - (buff + i)) + 1;
-#ifdef NETDEBUG
-                std::cout << thread_id  << "getLine result2 !SSL: " << result << std::endl;
-#endif
+                DEBUG_network("getLine result2 !SSL: ", result );
                 return i + (result - (buff + i));
             }
             i += tocopy;
@@ -457,10 +342,9 @@ int BaseSocket::getLine(char *buff, int size, int timeout, bool honour_reloadcon
         buff[i] = '\0';
         if (truncated)
             *truncated = true;
-#ifdef NETDEBUG
         if (truncated)
-            std::cout << thread_id  << "Getline(SSL) truncated buffer end reached before we found a newline: " << buff  << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;;
-#endif
+            DEBUG_network("Getline(SSL) truncated buffer end reached before we found a newline: ", buff );
+
         return i;
     } catch (...) {
         return -1;
@@ -468,63 +352,82 @@ int BaseSocket::getLine(char *buff, int size, int timeout, bool honour_reloadcon
 }
 
 // write line to socket
-bool BaseSocket::writeString(const char *line) //throw(std::exception)
+bool BaseSocket::writeString(const char *line)
 {
     int l = strlen(line);
     return writeToSocket(line, l, 0, timeout);
 }
 
-// write data to socket - throws exception on failure, can be told to break on config reloads
-//void BaseSocket::writeToSockete(const char *buff, int len, unsigned int flags, int timeout, bool honour_reloadconfig) throw(std::exception)
-void BaseSocket::writeToSockete(const char *buff, int len, unsigned int flags, int timeout, bool honour_reloadconfig)
-{
-    if (!writeToSocket(buff, len, flags, timeout, honour_reloadconfig)) {
-        throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
-    }
-}
-
-// write data to socket - can be told not to do an initial readyForOutput, and to break on config reloads
-bool BaseSocket::writeToSocket(const char *buff, int len, unsigned int flags, int timeout, bool check_first, bool honour_reloadconfig)
+bool BaseSocket::writeToSocket(const char *buff, int len, unsigned int flags, int timeout)
 {
     int actuallysent = 0;
     int sent;
     while (actuallysent < len) {
-        if (check_first) {
-//            try {
-                //readyForOutput(timeout, honour_reloadconfig); // throws exception on error or timeoutI/
-            //} catch (std::exception &e) {
-            //    return false;
-            //}
-            if(!breadyForOutput(timeout))
-                return false;
-        }
         sent = 0;
         s_errno = 0;
         errno = 0;
-        if(!isNoWrite()) sent = send(sck, buff + actuallysent, len - actuallysent, 0);
+        if(isNoWrite()) return false;
+        if((!isNoBlock) && doCheck) {
+            if( !readyForOutput(timeout)) {
+                s_errno = errno;
+                return false;
+            }
+        }
+        sent = send(sck, buff + actuallysent, len - actuallysent, 0);
 
-//        if (sent == 0)
-        if (sent  < 1)
-        {
+        if (sent  < 1) {
+            if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+                if(isNoBlock)
+                    if (readyForOutput(timeout)) continue; // now able to send
+            }
             s_errno = errno;
             return false; // other end is closed
         }
         actuallysent += sent;
     }
     return true;
+
+}
+
+// write data to socket - returns no of bytes written or 0 if would block and -1 on error
+int BaseSocket::writeToSocketNB(const char *buff, int len, unsigned int flags)
+{
+    int actuallysent = 0;
+    int sent;
+    while (actuallysent < len) {
+        sent = 0;
+        s_errno = 0;
+        errno = 0;
+        if(isNoWrite()) return -1;
+        if((!isNoBlock) && doCheck) {
+            if (!readyForOutput(0))
+                return 0;
+        }
+        sent = send(sck, buff + actuallysent, len - actuallysent, 0);
+
+            if (sent  < 1) {
+            if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+                s_errno = errno;
+                return 0;
+            }
+                s_errno = errno;
+                return -1; // other end is closed
+            }
+        actuallysent += sent;
+            return sent;
+    }
+    return -1;    //should never get here
 }
 
 // read a specified expected amount and return what actually read
-int BaseSocket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout)
+int BaseSocket::readFromSocket(char *buff, int len, unsigned int flags, int timeout, bool ret_part)
 {
     int cnt, rc;
     cnt = len;
 
     // first, return what's left from the previous buffer read, if anything
     if ((bufflen - buffstart) > 0) {
-#ifdef NETDEBUG
-        std::cout << thread_id  << "readFromSocketn: data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
-#endif
+        DEBUG_network("data already in buffer; bufflen: ", bufflen, " buffstart: ", buffstart);
         int tocopy = len;
         if ((bufflen - buffstart) < len)
             tocopy = bufflen - buffstart;
@@ -532,86 +435,48 @@ int BaseSocket::readFromSocketn(char *buff, int len, unsigned int flags, int tim
         cnt -= tocopy;
         buffstart += tocopy;
         buff += tocopy;
+        if (ret_part)
+            return tocopy;
         if (cnt == 0)
             return len;
     }
 
     while (cnt > 0) {
-//        try {
-//            checkForInput(timeout); // throws exception on error or timeout
-//        } catch (std::exception &e) {
-//            return -1;
-//        }
-//        if (isNoRead())  return -1;
-        if (!bcheckForInput(timeout))  return -1;
         s_errno = 0;
         errno = 0;
+        if((!isNoBlock) && doCheck) {
+            if(!checkForInput(timeout))
+                return -1;
+        }
         rc = recv(sck, buff, cnt, flags);
         if (rc < 0) {
-//i            if (errno == EINTR) {
-//                continue;
-//            }
+            if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+                if (checkForInput(timeout)) continue;// now got some input
+                else  {
+                    timedout = true;
+                    return -1;// timed out
+                }
+            }
             s_errno = errno;
             return -1;
         }
         if (rc == 0) { // eof
+            ishup = true;
             return len - cnt;
         }
-        buff += rc;
         cnt -= rc;
+        if(ret_part) return len - cnt;
+        buff += rc;
     }
     return len;
 }
 
-// read what's available and return error status - can be told not to do an initial checkForInput, and to break on reloads
-int BaseSocket::readFromSocket(char *buff, int len, unsigned int flags, int timeout, bool check_first, bool honour_reloadconfig)
-{
-    // first, return what's left from the previous buffer read, if anything
-    int cnt = len;
-    int tocopy = 0;
-    if ((bufflen - buffstart) > 0) {
-        tocopy = len;
 
-#ifdef NETDEBUG
-        std::cout << thread_id  << "readFromSocket: data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
-#endif
-        if ((bufflen - buffstart) < len)
-            tocopy = bufflen - buffstart;
-
-        memcpy(buff, buffer + buffstart, tocopy);
-        cnt -= tocopy;
-        buffstart += tocopy;
-        buff += tocopy;
-
-        if (cnt == 0)
-            return len;
+short int BaseSocket::get_wait_flag(bool write_flag) {
+    if (timedout)
+        timedout = false;
+    if (write_flag) {
+        return POLLOUT;
     }
-
-    int rc = 0;
-
-    if (check_first) {
-    //    try {
-    //        checkForInput(timeout, honour_reloadconfig);
-    //    } catch (std::exception &e) {
-    //        return -1;
-    //    }
-        if (! bcheckForInput(timeout))
-            return -1;
-    }
-    while (true) {
-        if (isNoRead())  return -1;
-        s_errno = 0;
-        errno = 0;
-        rc = recv(sck, buff, cnt, flags);
-        if (rc < 0) {
- //           if (errno == EINTR ) {
-//  ..             continue;
-//           }
-            s_errno = errno;
-            return -1;
-       }
-
-        break;
-    }
-    return rc + tocopy;
+    return POLLIN;
 }
