@@ -22,7 +22,8 @@ class ImageContainer
     // read image from file
     bool read(const char *filename);
     // send image to client
-    void display(Socket *s);
+   bool display(Socket *s);
+   bool display_hb(String &eheader, String &ebody);
 
     private:
     long int imagelength;
