@@ -7,14 +7,15 @@
 // INCLUDES
 
 #ifdef HAVE_CONFIG_H
-#include "dgconfig.h"
+#include "e2config.h"
 #endif
-#include <syslog.h>
-#include <algorithm>
 #include "ListContainer.hpp"
 #include "SBFunction.hpp"
 #include "OptionContainer.hpp"
 #include "RegExp.hpp"
+#include "Logger.hpp"
+
+#include <algorithm>
 #include <cstdlib>
 #include <cstdio>
 #include <ctime>
@@ -28,32 +29,13 @@
 
 // GLOBALS
 
-extern bool is_daemonised;
 extern OptionContainer o;
-extern thread_local std::string thread_id;
 
 // DEFINES
 
 // Constructor - set default values
 SBFunction::SBFunction()
 {
-#ifdef NOT_DEF
-    bi_funct_map = {
-            "setexception",
-            "setgrey",
-            "setblock",
-            "setmodurl",
-            "setdone",
-            "true",
-            "false",
-            "setgomitm",
-            "setlogcategory",
-            "setaddheader",
-            "setredirect",
-            "setnocheckcert",
-            "setsearchterm"
-    };
-#endif
 }
 
 // delete the memory block when the class is destryed
@@ -88,22 +70,19 @@ bool SBFunction::addline(String command, String params, String action, unsigned
         rec.action_id = SB_COM_IFNOT;
         rec.isif = false;
     } else {
-        std::cout << thread_id << "StoryBoard error: Invalid command " << command << "at line " << line_no << " of " << file_name << std::endl;
+        E2LOGGER_error("StoryBoard error: Invalid command ", command, "at line ", String(line_no), " of ", file_name);
         return false;
     }
     rec.file_lineno = line_no;
     // process params
-#ifdef DGDEBUG
-    std::cerr << thread_id << "CLine " << params << " Action " << action << std::endl;
-#endif
+    DEBUG_story("CLine ", params, " Action ", action);
+
     String state;
     String temp;
     String temp2;
     if (params.contains(",")) {
         state = params.before(",");
-#ifdef DGDEBUG
-        std::cerr << thread_id << "CLine state is " << state << std::endl;
-#endif
+        DEBUG_story("CLine state is ", state);
         temp = params.after(",");
     } else {
         state = params;
@@ -111,7 +90,7 @@ bool SBFunction::addline(String command, String params, String action, unsigned
     state.removeWhiteSpace();
     rec.state = getStateID(state);
     if (rec.state == 0) {
-        std::cout << thread_id << "StoryBoard error: Invalid state " << state << " at line " << line_no << " of " << file_name << std::endl;
+        E2LOGGER_error("StoryBoard error: Invalid state ", state, " at line ", String(line_no), " of ", file_name);
         return false;
     }
 
@@ -125,10 +104,8 @@ bool SBFunction::addline(String command, String params, String action, unsigned
         }
         temp2.removeWhiteSpace();
         rec.list_name = temp2;
-#ifdef DGDEBUG
-            std::cerr << thread_id << "CLine list is " << temp2 << std::endl;
-#endif
-     }
+        DEBUG_story("CLine list is ", temp2);
+    }
     if ( temp.length() ) {
         if (temp.contains(",")) {
             temp2 = temp.before(",");
@@ -139,9 +116,7 @@ bool SBFunction::addline(String command, String params, String action, unsigned
         }
         temp2.removeWhiteSpace();
         rec.mess_no = temp2.toInteger();
-#ifdef DGDEBUG
-        std::cerr << thread_id << "CLine mno is " << temp2 << std::endl;
-#endif
+        DEBUG_story("CLine mess_no is ", temp2);
     }
     if ( temp.length() ) {
         if (temp.contains(",")) {
@@ -153,9 +128,7 @@ bool SBFunction::addline(String command, String params, String action, unsigned
         }
         temp2.removeWhiteSpace();
         rec.log_mess_no = temp2.toInteger();
-#ifdef DGDEBUG
-        std::cerr << thread_id << "CLine logmno is " << temp2 << std::endl;
-#endif
+        DEBUG_story("CLine log_mess_no is ", temp2);
     }
     if ( temp.length() ) {
         if (temp.contains(",")) {
@@ -168,9 +141,7 @@ bool SBFunction::addline(String command, String params, String action, unsigned
         temp2.removeWhiteSpace();
         if (temp2 == "optional")
             rec.optional = true;
-#ifdef DGDEBUG
-        std::cerr << thread_id << "CLine optional is true" << std::endl;
-#endif
+        DEBUG_story("CLine optional is true");
     }
     // check list and get list_ID - needs ListMeta object - done in StoryBook::readfile
 
@@ -181,8 +152,8 @@ bool SBFunction::addline(String command, String params, String action, unsigned
         rec.return_after_action = true;
         action = action.after("return ");
     } else if (action.startsWith("returnif ")) {
-                rec.return_after_action_is_true = true;
-                action = action.after("returnif ");
+        rec.return_after_action_is_true = true;
+        action = action.after("returnif ");
     };
 
     action.removeWhiteSpace();
@@ -205,11 +176,8 @@ unsigned int SBFunction::getStateID(String & state) {
 
 unsigned int SBFunction::getBIFunctID(String &action)  {    // get built-in function id
     unsigned int i = 0;
-  //  std::cerr << thread_id << "getBIFuctID looking for " << action << " in map_size " << bi_funct_map.size() << std::endl;
     while (i < SB_FUNC_MAP_SIZE) {
-  //      std::cerr << thread_id << "getBIFuctID checking index " << i << " " << bi_funct_map.at(i) << std::endl;
         if ( action == bi_funct_map.at(i)) {
-    //        std::cerr << thread_id << "getBIFuctID l;ooking for " << action << std::endl;
             return ++i + SB_BI_FUNC_BASE;
         }
         ++i;
@@ -220,7 +188,8 @@ unsigned int SBFunction::getBIFunctID(String &action)  {    // get built-in func
 String SBFunction::getState(unsigned int id) {     // get condition statement from state_id
     if (--id < SB_STATE_MAP_SIZE)
         return state_map[id];
-    return "";
+    else
+        return String(id);
 };
 
 String SBFunction::getBIFunct(unsigned int &id) {    // get built-in function (action) from funct_id
@@ -229,7 +198,7 @@ String SBFunction::getBIFunct(unsigned int &id) {    // get built-in function (a
         if (i < SB_FUNC_MAP_SIZE)
             return bi_funct_map[i];
     }
-    return "";
+    return String(id);
 };
 
 String SBFunction::getName() {
