@@ -301,7 +301,7 @@ void String::fullPath(String &base_dir) {
 }
 
 // decode %xx to individual characters (checkme: i'm sure this is duplicated elsewhere...)
-void String::hexDecode()
+void String::hexDecode()            //TODO: make this more efficient - use similar login to hex2integer
 {
     if (this->length() < 3)
         return;
@@ -751,7 +751,9 @@ bool String::isIp() {
 }
 
 bool String::is_valid_domain() {
+    if (this->empty()) return false;
     unsigned int l = this->length();
+    if(l < 4) return false;  // too short - can not possibly be a valid domain!
     const char *d = this->c_str();
     if ((d[0] == '-')||(d[l - 1] == '-')) return false;  // hyphens are not allowed at start or end of domain
     for (unsigned int i = 0; i < l; i++) {
