@@ -4,11 +4,10 @@
 // http://e2guardian.org/
 // Released under the GPL v2, with the OpenSSL exception described in the README file.
 
-//TODO: Replace error reporting with detailed entries in syslog(LOG_ERR), short entries in lastmessage.
 
 // INCLUDES
 #ifdef HAVE_CONFIG_H
-#include "dgconfig.h"
+#include "e2config.h"
 #endif
 
 #include "../String.hpp"
@@ -16,8 +15,8 @@
 #include "../ContentScanner.hpp"
 #include "../UDSocket.hpp"
 #include "../OptionContainer.hpp"
+#include "../Logger.hpp"
 
-#include <syslog.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -26,8 +25,6 @@
 // GLOBALS
 
 extern OptionContainer o;
-extern bool is_daemonised;
-extern thread_local std::string thread_id;
 
 // IMPLEMENTATION
 
@@ -63,15 +60,13 @@ CSPlugin *kavdcreate(ConfigVar &definition)
 int kavdinstance::init(void *args)
 {
     int rc;
-    if ((rc = CSPlugin::init(args)) != DGCS_OK)
+    if ((rc = CSPlugin::init(args)) != E2CS_OK)
         return rc;
 
     udspath = cv["kavdudsfile"];
     if (udspath.length() < 3) {
-        if (!is_daemonised)
-            std::cerr << thread_id << "Error reading kavdudsfile option." << std::endl;
-        syslog(LOG_ERR, "%s", "Error reading kavdudsfile option.");
-        return DGCS_ERROR;
+        E2LOGGER_error("Error reading kavdudsfile option.");
+        return E2CS_ERROR;
         // it would be far better to do a test connection to the file but
         // could not be arsed for now
     }
@@ -79,7 +74,7 @@ int kavdinstance::init(void *args)
     // read in path prefix
     pathprefix = cv["pathprefix"];
 
-    return DGCS_OK;
+    return E2CS_OK;
 }
 
 // no need to replace the inheritied scanMemory() which just calls scanFile()
@@ -97,8 +92,8 @@ int kavdinstance::scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, con
     // the AV daemon's group membership.
     // chmod can error with EINTR, ignore this?
     if (chmod(filename, S_IRGRP | S_IRUSR) != 0) {
-        syslog(LOG_ERR, "Could not change file ownership to give kavd read access: %s", strerror(errno));
-        return DGCS_SCANERROR;
+        E2LOGGER_error("Could not change file ownership to give kavd read access: ", strerror(errno));
+        return E2CS_SCANERROR;
     };
     String command("SCAN bPQRSTUW ");
     if (pathprefix.length()) {
@@ -108,60 +103,56 @@ int kavdinstance::scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, con
         command += filename;
     }
     command += "\r\n";
-#ifdef DGDEBUG
-    std::cerr << thread_id << "kavdscan command:" << command << std::endl;
-#endif
+    DEBUG_avscan("kavdscan command:", command);
+
     UDSocket stripedsocks;
     if (stripedsocks.getFD() < 0) {
-        syslog(LOG_ERR, "%s", "Error creating socket for talking to kavdscan");
-        return DGCS_SCANERROR;
+        E2LOGGER_error("Error creating socket for talking to kavdscan");
+        return E2CS_SCANERROR;
     }
     if (stripedsocks.connect(udspath.toCharArray()) < 0) {
-        syslog(LOG_ERR, "%s", "Error connecting to kavdscan socket");
+        E2LOGGER_error("Error connecting to kavdscan socket");
         stripedsocks.close();
-        return DGCS_SCANERROR;
+        return E2CS_SCANERROR;
     }
     char *buff = new char[4096];
     memset(buff, 0, 4096);
     int rc;
     try {
         // read kaspersky kavdscan (AV Enging Server) - format: 2xx greeting
-        rc = stripedsocks.getLine(buff, 4096, o.content_scanner_timeout);
+        rc = stripedsocks.getLine(buff, 4096, o.content.content_scanner_timeout);
     } catch (std::exception &e) {
     }
     if (buff[0] != '2') {
         delete[] buff;
         stripedsocks.close();
-        syslog(LOG_ERR, "%s", "kavdscan did not return ok");
-        return DGCS_SCANERROR;
+        E2LOGGER_error("kavdscan did not return ok");
+        return E2CS_SCANERROR;
     }
     try {
         stripedsocks.writeString(command.toCharArray());
     } catch (std::exception &e) {
         delete[] buff;
         stripedsocks.close();
-        syslog(LOG_ERR, "%s", "unable to write to kavdscan");
-        return DGCS_SCANERROR;
+        E2LOGGER_error("unable to write to kavdscan");
+        return E2CS_SCANERROR;
     }
     try {
-        rc = stripedsocks.getLine(buff, 4096, o.content_scanner_timeout);
+        rc = stripedsocks.getLine(buff, 4096, o.content.content_scanner_timeout);
     } catch (std::exception &e) {
         delete[] buff;
         stripedsocks.close();
-        syslog(LOG_ERR, "%s", "Error reading kavdscan socket");
-        return DGCS_SCANERROR;
+        E2LOGGER_error("Error reading kavdscan socket");
+        return E2CS_SCANERROR;
     }
     String reply(buff);
-#ifdef DGDEBUG
-    std::cerr << thread_id << "Got from kavdscan:" << reply << std::endl;
-#endif
+    DEBUG_avscan("Got from kavdscan:", reply);
+
     if (reply[0] == '2') { // clean
-#ifdef DGDEBUG
-        std::cerr << thread_id << "kavdscan - clean" << std::endl;
-#endif
+        DEBUG_avscan("kavdscan - clean");
         delete[] buff;
         stripedsocks.close();
-        return DGCS_CLEAN;
+        return E2CS_CLEAN;
     }
     if (reply.startsWith("322")) { // infected
         // patch to handle multiple virii in kavd response
@@ -170,29 +161,27 @@ int kavdinstance::scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, con
             reply.removeWhiteSpace();
             lastvirusname = lastvirusname + " " + reply.after("322-").before(" ");
             try {
-                rc = stripedsocks.getLine(buff, 4096, o.content_scanner_timeout);
+                rc = stripedsocks.getLine(buff, 4096, o.content.content_scanner_timeout);
             } catch (std::exception &e) {
                 delete[] buff;
                 stripedsocks.close();
-                syslog(LOG_ERR, "%s", "Error reading kavdscan socket");
-                return DGCS_SCANERROR;
+                E2LOGGER_error("Error reading kavdscan socket");
+                return E2CS_SCANERROR;
             }
             reply = buff;
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Got from kavdscan:" << reply << std::endl;
-#endif
+            DEBUG_avscan("Got from kavdscan:", reply);
         }
-        std::cerr << thread_id << "lastvirusname: " << lastvirusname << std::endl;
+        E2LOGGER_error("lastvirusname: ", lastvirusname);
         delete[] buff;
         stripedsocks.close();
 
         // format: 322 nastyvirus blah
         blockFile(NULL, NULL, checkme);
-        return DGCS_INFECTED;
+        return E2CS_INFECTED;
     }
     delete[] buff;
     stripedsocks.close();
     // must be an error then
     lastmessage = reply;
-    return DGCS_SCANERROR;
+    return E2CS_SCANERROR;
 }
